function [grid_pts, Xgrid] = Extract_Corners(img);


% [grid_pts, Xgrid] = Extract_Corners(I,map);
% 
% Extracts all the grid points on the image I at subpixel accuracy, and
% returns also the 3D coordinates of the corresponding points in the object
% reference frame.
%
% Inputs: img: image
%
% Outputs: grid_pts: the extracted corner positions (2xNp) matrix
%                    The first row -> the x coordinates
%                    The second row -> the y coordinates
%                    Note that we start indexing the row and columns by (0,0)
%                    This is different from matlab which in starts
%                    indexing at 1 (and not 0)
%           Xgrid: Coordinates in the object frame of the corresponding points
%                  A 2xNp matrix where the first row are the x coordinates,
%                  the second row, the y coordinates, and the third row the z
%                  coordinates.


% California Institute of Technology
% (c) Jean-Yves Bouguet -- March 4th, 1997


C_code = 1;

if (nargin < 1),
   error('Are you sure you want to do calibration without an image?');
end;

wintx = 3; % neigborhood of integration for
winty = 3; % the corner finder



[ny,nx] = size(img);

figure(1);
imshow(img);



title('Click on the four extreme corners of the complete regular pattern...');

disp('Click on the four extreme corners of the complete regular pattern...');

[x,y] = ginput(4);



%load corners;

Xc = cornerfinder([x';y'],img,wintx,winty); % the four corners

x = Xc(1,:)';
y = Xc(2,:)';

[y,indy] = sort(y);
x = x(indy);

if (x(2) > x(1)),
   x4 = x(1);y4 = y(1); x3 = x(2); y3 = y(2);
else
   x4 = x(2);y4 = y(2); x3 = x(1); y3 = y(1);
end;
if (x(3) > x(4)),
   x2 = x(3);y2 = y(3); x1 = x(4); y1 = y(4);
else
   x2 = x(4);y2 = y(4); x1 = x(3); y1 = y(3);
end;

x = [x1;x2;x3;x4];
y = [y1;y2;y3;y4];


figure(1); hold on;
plot(x,y,'r+');
hold off;

n_sq_x = input('Number of square on a row [11] = '); %6

if n_sq_x == [], n_sq_x = 11; end;

n_sq_y = input('Number of square on a column [9] = '); %6

if n_sq_y == [], n_sq_y = 9; end;


[XX] = projectedGrid ( [x(1);y(1)], [x(2);y(2)],[x(3);y(3)], [x(4);y(4)],n_sq_x+1,n_sq_y+1); % The full grid


Np = (n_sq_x+1)*(n_sq_y+1);



grid_pts = cornerfinder(XX,img,wintx,winty); 		%%% Finds the exact corners at every points!

if C_code,
   grid_pts = grid_pts - 1;
end;

%grid_pts = XX; % no corner finder

ind_corners = [1 n_sq_x+1 (n_sq_x+1)*n_sq_y+1 (n_sq_x+1)*(n_sq_y+1)]; % index of the 4 corners
ind_orig = (n_sq_x+1)*n_sq_y + 1;
xorig = grid_pts(1,ind_orig);
yorig = grid_pts(2,ind_orig);
dxpos = mean([grid_pts(:,ind_orig) grid_pts(:,ind_orig+1)]');
dypos = mean([grid_pts(:,ind_orig) grid_pts(:,ind_orig-n_sq_x-1)]');


figure(1);
image(img);hold on;
plot(grid_pts(1,:)+1,grid_pts(2,:)+1,'r+');
plot(grid_pts(1,ind_corners)+1,grid_pts(2,ind_corners)+1,'mo');
plot(xorig+1,yorig+1,'*m');
h = text(xorig-15,yorig-15,'O');
set(h,'Color','m','FontSize',14);
h2 = text(dxpos(1)-10,dxpos(2)-10,'dX');
set(h2,'Color','g','FontSize',14);
h3 = text(dypos(1)-25,dypos(2)-3,'dY');
set(h3,'Color','g','FontSize',14);
xlabel('Xc (in camera frame)');
ylabel('Yc (in camera frame)');
hold off;



dX = input('Size dX in x of the squares [1.9912cm] = ');
dY = input('Size dY in y of the squares  [2.0139cm] = ');
if dX == [], dX = 21.9250/11; end;
if dY == [], dY =  18.1250/9; end;



%dX = 19.9/10; 				%%% Size along x
%dY = 16.1/8; 				%%% Size along y



Xi = reshape(([0:n_sq_x]*dX)'*ones(1,n_sq_y+1),Np,1)';
Yi = reshape(ones(n_sq_x+1,1)*[n_sq_y:-1:0]*dY,Np,1)';
Zi = zeros(1,Np);


Xgrid = [Xi;Yi;Zi];

